import {Button, PageContainer, ScrollView, Swiper, SwiperItem, View} from "@tarojs/components";
import {FC, useEffect, useState} from "react";
import HVideo from "@/components/video/video";
import {CurEndParam, curriculum, HourPlayData} from "@/api";
import {Profile} from '@/store'
import Taro from "@tarojs/taro";
import Multi from "@/components/topic/multi";
import Judge from "@/components/topic/judge";
import ShortAnswer from "@/components/topic/shortAnswer";
import unique_ident from "@/hooks/unique_ident";

interface Props {
  id: number,
  courseId: number
  preview?: boolean
  curEnd: () => void
}


let seek: (time: number) => void
let record: boolean[] = [] //答案记录
const Course: FC<Props> = ({id, courseId, preview, curEnd}: Props) => {
  const [breakpoint, setBreakpoint] = useState<number[]>([])   // 断点
  const [show, setShow] = useState(false)  // 题
  const [data, setData] = useState<HourPlayData | null>(null)
  const [examAll, setExamAll] = useState<Record<number, (ShareSubject | Multi)[]>>([])  // 题库
  const [index, setIndex] = useState(0)
  const [time, setTime] = useState<number>(0) // 进入断点的时间
  const [validate, setValidate] = useState(false) // 开启验证
  const [newRecord, setNewRecord] = useState<boolean[]>([])
  const [count, setCount] = useState<Record<number, number> | null>(null)
  const [frequency, setFrequency] = useState<number>(1) // 次数
  const [testId, setTestId] = useState<number | null>(null)
  const [startRecording, setStartRecording] = useState<CurEndParam | null>(null) // 开始记录
  const {user} = Profile.useContainer()


  async function onEnded() {
    unique_ident.put(data?.duration, Date.now())  // 记录

    startRecording && curriculum.curEnd(courseId, id, {...startRecording, duration: data?.duration!}) //结束
    if (testId) {
      Taro.navigateTo({
        url: `/pages/business/test/test?testId=${testId}`
      })
    } else {
      Taro.showModal({
        title: "学习完成",
        success() {
          curEnd()
        }
      })
    }
  }

  /** 进入断点 */
  function onBreakpoint(breakpoint: number) {
    if (breakpoint !== time) {
      setFrequency(count?.[breakpoint] || 1)
    }
    setTime(breakpoint)
    setShow(true)
  }

  async function getData() {
    unique_ident.put()

    const res = await curriculum.hourPlay(courseId, id)

    if (res) {
      setData(res)
      setBreakpoint(res.timeList)
      setExamAll(res.hourExamQuestions || [])
      setCount(res.count)
      if (res.hour_test) {
        setTestId(res.hour_test.id)
      }

      setStartRecording({
        duration: 0,
        unique_ident: res.unique_ident,
        start_date: Date.now()
      })

      unique_ident.set({
        courseId: Number(courseId),
        user_id: user?.id!,
        duration: 0,
        start_date: Date.now(),
        end_date: Date.now(),
        hour_id: id,
        unique_ident: res.unique_ident
      })
    }
  }

  useEffect(() => {
    init()
    getData()
  }, [id])

  /** 统计答案 */
  function onAnswer(isAnswer: boolean) {
    record.push(isAnswer)
    setNewRecord(record)
  }

  function init() {
    record = []
    setTime(0)
    setShow(false)
    setIndex(0)
    setValidate(false)
    setNewRecord([])
  }

  /** 再来一次 */
  function onceMore() {
    const oldTime: number = time
    init()
    setTimeout(() => {
      setShow(true)
      setTime(oldTime)
    })
  }

  /** 重新看 */
  function again() {
    if (frequency === 0 && show) {
      const {id: question_id, question_type} = examAll?.[time]?.[0]
      curriculum.answerRecord(id, {
        is_pass: false,
        user_id: user?.id!,
        time: time,
        question_id,
        question_type
      }).then(res => {
        seek(res.time)
      })
      init()
    }
    setShow(false)
    setIndex(0)
  }

  useEffect(() => {
    if (!newRecord.length) return;
    setFrequency(frequency - 1)
    const pass = newRecord.every(d => d)
    if (pass) {
      const {id: question_id, question_type} = examAll?.[time]?.[0]
      curriculum.answerRecord(id, {
        is_pass: pass,
        user_id: user?.id!,
        time: time,
        question_type,
        question_id
      }).then()
      const old: number[] = JSON.parse(JSON.stringify(breakpoint))
      const index = old.indexOf(time)
      old.splice(index, 1)
      setBreakpoint(old)
      Taro.showToast({title: '答题正确', duration: 1000})
      seek(time)
      init()
    } else {
      Taro.showToast({title: '错误', icon: 'error'})
    }
  }, [newRecord])

  function videoSeek(fn: (time: number) => void) {
    seek = fn
  }


  return (
    <>
      <HVideo
        duration={data?.duration || 0}
        preview={!!preview}
        src={data?.url || ''}
        onEnded={onEnded}
        breakpoint={breakpoint}
        onBreakpoint={onBreakpoint}
        setTime={videoSeek}
      />

      <view>
        <PageContainer
          show={show}
          position='bottom'
          round
          onAfterLeave={again}
        >
          <Swiper style={{height: "70vh"}} snapToEdge current={index}>
            {examAll?.[time]?.map((d, index) =>
              <SwiperItem>
                <ScrollView style='height:70vh' scrollY>
                  {d.question_type === 1 &&
                    <Multi
                      frequency={frequency}
                      data={d as Multi}
                      onAnswer={onAnswer}
                      onUpAndDown={(index) => setIndex(index)}
                      index={index}
                      validate={validate}
                    />}
                  {d.question_type === 2 &&
                    <Judge
                      frequency={frequency}
                      onUpAndDown={(index) => setIndex(index)}
                      index={index}
                      data={d as ShareSubject}
                      validate={validate}
                      onAnswer={onAnswer}
                    />}
                  {d.question_type === 3 &&
                    <ShortAnswer
                      frequency={frequency}
                      data={d as ShareSubject}
                      onAnswer={onAnswer}
                      onUpAndDown={(index) => setIndex(index)}
                      index={index}
                      validate={validate}
                    />}
                </ScrollView>
              </SwiperItem>
            )}

            <SwiperItem>
              <ScrollView style='height:70vh' scrollY>
                <View className='mt-10 ml-4'>
                  <View className='mt-3'>正确：{newRecord.filter(d => d).length}</View>
                  <View className='mt-3'>错误：{newRecord.filter(d => !d).length}</View>
                  <View className='mt-3'>剩余答题次数{frequency}</View>
                </View>

                {!validate && <Button className='button mt-10' onClick={() => setValidate(true)}>提交</Button>}
                {frequency > 0 && validate && <Button className='button mt-10' onClick={onceMore}>再来一次</Button>}
                {frequency === 0 && validate && <Button className='button mt-10' onClick={again}>重新学习</Button>}
              </ScrollView>
            </SwiperItem>
          </Swiper>
          <View className='statistics'>
            <View>共{examAll?.[time]?.length}题</View>
          </View>
        </PageContainer>
      </view>
    </>

  )
}
export default Course
