import {FC, useEffect, useRef, useState} from "react";
import {Profile} from "@/store";
import {Button, Form, Image, Input, PageContainer, Text, View} from "@tarojs/components";
import Taro from "@tarojs/taro";
import styles from './login.module.scss'
import Loading from "@/components/loading";
import Icon from "@/components/icon";
import {userApi} from "@/api";
import {regexTel} from "@/utils/regu";

interface BingProps {
  code: string
  catch_key: string
}

const Bing: FC<BingProps> = ({code, catch_key}: BingProps) => {
  const [useCode, setUseCode] = useState<string>(code)
  const form = useRef<HTMLFormElement | null>(null)
  const [loading, setLoading] = useState(false)
  const {setUser, setToken, setCompany} = Profile.useContainer()

  useEffect(() => {
    form.current?.reset?.()
    setUseCode(code)
  }, [code])

  async function refreshCode() {
    try {
      const {code} = await userApi.code(catch_key)
      setUseCode(code.image)
    } catch (e) {
    }
  }

  async function Submit(data) {
    Taro.showLoading()
    setLoading(true)
    const value = data.target.value
    if (!regexTel.exec(value.phone_number)) {
      Taro.showToast({title: '手机号错误', icon: 'error'})
      setLoading(false)
      return
    }
    try {
      const res = await userApi.checkout({...value, catch_key})
      if (res) {
        setCompany(res.company)
        setUser(res.user)
        setToken(res.token)
        Taro.switchTab({url: '/pages/index/index'})
      }
    } catch (e) {
      refreshCode()
    }
    Taro.hideLoading()
    setLoading(false)
  }

  return (
    <View className='h-5 pt-6 px-3'>
      <Form className='form' onSubmit={Submit} ref={form}>
        <View className='item'>
          <View className='label'>手机号</View>
          <Input name='phone_number' placeholder={'请输入手机号'}/>
        </View>

        <View className='item'>
          <View className='label'>验证码</View>
          <View className='flex align-center'>
            <Input name='code' className='flex-1' placeholder={'请输入验证码'}/>
            <Image className='w-2 ml-1' style='height:28px' src={useCode} onClick={refreshCode}/>
          </View>
        </View>

        <Button className={'button ' + styles.submit} style='margin:30px auto' formType='submit'
                disabled={loading}>提交</Button>
      </Form>
    </View>
  )
}

const Login = () => {
  const {statusBarHeight = 0} = Taro.getSystemInfoSync()
  const bbc = Taro.getMenuButtonBoundingClientRect();
  const navHeight = bbc.bottom + (bbc.top - statusBarHeight) - statusBarHeight


  const [isLoading, setLoading] = useState(false)
  const [error, setError] = useState<string | null>(null)
  const [validateCode, setCode] = useState<string | null>(null)
  const [catch_key, setCatch_key] = useState<string | null>(null)
  const {setUser, setToken, setCompany} = Profile.useContainer()


  function login() {
    if (isLoading) return;
    setLoading(true)
    Taro.login({
      success: async (res) => {
        try {
          const {code, catch_key, user, token, company} = await userApi.login(res.code)
          if (!code) {
            setUser(user)
            setToken(token)
            setCompany(company)
            Taro.switchTab({url: '/pages/index/index'})
            return
          }
          setCatch_key(catch_key)
          setCode(code.image)
        } catch (e) {
        }
        setLoading(false)
      },
      fail: (res) => {
        setError(res.errMsg)
        setLoading(false)
      },
    })
  }

  return (
    <View className={styles.container}>
      <PageContainer show={!!validateCode} position='bottom' round onBeforeLeave={() => setCode(null)}>
        {validateCode && <Bing code={validateCode!} catch_key={catch_key!}/>}
      </PageContainer>
      <View className={styles.navbar} style={`height:${navHeight}px;margin-top:${statusBarHeight}px`}>
        <Text>微信授权登录</Text>
        {error ? <View className={styles.errorTips}>
          <View style={{flex: 1}}>{error}</View>
          <View>
            <Icon name={'close'} onClick={() => setError(null)}/>
          </View>
        </View> : null}
      </View>
      <View className={styles.brand}>
        <Image mode={'scaleToFill'} src="https://admin.playedu.xyz/favicon.ico"/>
      </View>
      <View className={styles.loginTips}>
        <Text>请完成微信授权以继续使用！</Text>
      </View>
      <Button className={'button ' + styles.submit} onClick={login} disabled={isLoading}>
        {isLoading ? <Loading/> : null}
        <Text>微信授权登录</Text>
      </Button>
    </View>
  )
}

const Index: FC = () => {
  return (
    <Profile.Provider>
      <Login/>
    </Profile.Provider>
  )
}
export default Index
